@@ -20,7 +20,7 @@ def __get_version() -> Optional[str]:
     return None
 
 __version__ = __get_version() or "0.0.0"
-__cryptonym__ = "baseless allegation"
+__cryptonym__ = "preposterous ports"
 
 # Add the devtools debug() function to builtins if available
 import builtins
