@@ -23,6 +23,7 @@ import servo.events
 import servo.logging
 import servo.pubsub
 import servo.repeating
+import servo.telemetry
 import servo.utilities.associations
 from servo.types import *
 
@@ -114,6 +115,10 @@ class BaseConnector(
         """The optimizer for the connector."""
         return self.config.optimizer
 
+    ##
+    # Shared telemetry metadata
+    telemetry: servo.telemetry.Telemetry = pydantic.Field(default_factory=servo.telemetry.Telemetry)
+
     ##
     # Validators
 
