@@ -36,6 +36,7 @@ PROMETHEUS_ANNOTATION_DEFAULTS = {
     "prometheus.opsani.com/port": "9901",
 
 }
+ENVOY_SIDECAR_IMAGE_TAG = 'opsani/envoy-proxy:v1.19-latest'
 ENVOY_SIDECAR_LABELS = {
     "sidecar.opsani.com/type": "envoy"
 }
@@ -56,7 +57,9 @@ class OpsaniDevConfiguration(servo.BaseConfiguration):
     port: Optional[Union[pydantic.StrictInt, str]] = None
     cpu: CPU
     memory: Memory
+    static_environment_variables: Optional[Dict[str, str]]
     prometheus_base_url: str = PROMETHEUS_SIDECAR_BASE_URL
+    envoy_sidecar_image: str = ENVOY_SIDECAR_IMAGE_TAG
     timeout: servo.Duration = "5m"
     settlement: Optional[servo.Duration] = pydantic.Field(
         description="Duration to observe the application after an adjust to ensure the deployment is stable. May be overridden by optimizer supplied `control.adjust.settlement` value."
@@ -107,6 +110,7 @@ class OpsaniDevConfiguration(servo.BaseConfiguration):
                     alias="main",
                     cpu=self.cpu,
                     memory=self.memory,
+                    static_environment_variables=self.static_environment_variables,
                 )
             ],
         )
@@ -168,24 +172,24 @@ class OpsaniDevConfiguration(servo.BaseConfiguration):
                 servo.connectors.prometheus.PrometheusMetric(
                     "main_success_rate",
                     servo.types.Unit.requests_per_second,
-                    query='avg(rate(envoy_cluster_upstream_rq_xx{opsani_role!="tuning", envoy_response_code_class=~"2|3"}[1m]))',
+                    query='avg(sum by(kubernetes_pod_name)(rate(envoy_cluster_upstream_rq_xx{opsani_role!="tuning", envoy_response_code_class=~"2|3"}[1m])))',
                 ),
                 servo.connectors.prometheus.PrometheusMetric(
                     "tuning_success_rate",
                     servo.types.Unit.requests_per_second,
-                    query='avg(rate(envoy_cluster_upstream_rq_xx{opsani_role="tuning", envoy_response_code_class=~"2|3"}[1m]))',
+                    query='avg(sum by(kubernetes_pod_name)(rate(envoy_cluster_upstream_rq_xx{opsani_role="tuning", envoy_response_code_class=~"2|3"}[1m])))',
                     absent=servo.connectors.prometheus.AbsentMetricPolicy.zero
                 ),
                 servo.connectors.prometheus.PrometheusMetric(
                     "main_error_rate",
                     servo.types.Unit.requests_per_second,
-                    query='avg(rate(envoy_cluster_upstream_rq_xx{opsani_role!="tuning", envoy_response_code_class=~"4|5"}[1m]))',
+                    query='avg(sum by(kubernetes_pod_name)(rate(envoy_cluster_upstream_rq_xx{opsani_role!="tuning", envoy_response_code_class=~"4|5"}[1m])))',
                     absent=servo.connectors.prometheus.AbsentMetricPolicy.zero
                 ),
                 servo.connectors.prometheus.PrometheusMetric(
                     "tuning_error_rate",
                     servo.types.Unit.requests_per_second,
-                    query='avg(rate(envoy_cluster_upstream_rq_xx{opsani_role="tuning", envoy_response_code_class=~"4|5"}[1m]))',
+                    query='avg(sum by(kubernetes_pod_name)(rate(envoy_cluster_upstream_rq_xx{opsani_role="tuning", envoy_response_code_class=~"4|5"}[1m])))',
                     absent=servo.connectors.prometheus.AbsentMetricPolicy.zero
                 ),
                 servo.connectors.prometheus.PrometheusMetric(
@@ -256,6 +260,13 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
     ]) -> Dict[str, str]:
         ...
 
+    @abc.abstractmethod
+    def _get_controller_patch_target(self, controller: Union[
+        servo.connectors.kubernetes.Deployment,
+        servo.connectors.kubernetes.Rollout
+    ]) -> str:
+        ...
+
     ##
     # Kubernetes essentials
 
@@ -300,15 +311,15 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
                         ), f'Not allowed to "{verb}" resource "{resource}"'
 
     @servo.checks.require('Namespace "{self.config.namespace}" is readable')
-    async def check_kubernetes_namespace(self) -> None:
+    async def check_opsani_dev_kubernetes_namespace(self) -> None:
         await servo.connectors.kubernetes.Namespace.read(self.config.namespace)
 
     @servo.checks.require('{self.controller_type_name} "{self.config_controller_name}" is readable')
-    async def check_kubernetes_controller(self) -> None:
+    async def check_opsani_dev_kubernetes_controller(self) -> None:
         await self.controller_class.read(self.config_controller_name, self.config.namespace)
 
     @servo.checks.require('Container "{self.config.container}" is readable')
-    async def check_kubernetes_container(self) -> None:
+    async def check_opsani_dev_kubernetes_container(self) -> None:
         controller = await self.controller_class.read(self.config_controller_name, self.config.namespace)
         container = controller.find_container(self.config.container)
         assert (
@@ -388,11 +399,11 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
             raise RuntimeError(f'{self.controller_type_name} "{controller.name}" is not ready')
 
     @servo.checks.require("service")
-    async def check_kubernetes_service(self) -> None:
+    async def check_opsani_dev_kubernetes_service(self) -> None:
         await servo.connectors.kubernetes.Service.read(self.config.service, self.config.namespace)
 
     @servo.checks.warn("service type")
-    async def check_kubernetes_service_type(self) -> None:
+    async def check_opsani_dev_kubernetes_service_type(self) -> None:
         service = await servo.connectors.kubernetes.Service.read(
             self.config.service, self.config.namespace
         )
@@ -403,7 +414,7 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
             )
 
     @servo.checks.check("service port")
-    async def check_kubernetes_service_port(self) -> None:
+    async def check_opsani_dev_kubernetes_service_port(self) -> None:
         service = await servo.connectors.kubernetes.Service.read(
             self.config.service, self.config.namespace
         )
@@ -592,8 +603,9 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
             annotations = dict(map(lambda k: (k, required_annotations[k]), delta))
             patch = {"spec": {"template": {"metadata": {"annotations": annotations}}}}
             patch_json = json.dumps(patch, indent=None)
+            controller_patch_target = self._get_controller_patch_target(controller)
             # NOTE: custom resources don't support strategic merge type. json merge is acceptable for both cases because the patch json doesn't contain lists
-            command = f"kubectl --namespace {self.config.namespace} patch {self.controller_type_name} {self.config_controller_name} --type='merge' -p '{patch_json}'"
+            command = f"kubectl --namespace {self.config.namespace} patch {controller_patch_target} --type='merge' -p '{patch_json}'"
             desc = ', '.join(sorted(delta))
             raise servo.checks.CheckError(
                 f"{self.controller_type_name} '{controller.name}' is missing annotations: {desc}",
@@ -618,8 +630,9 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
             desc = ', '.join(sorted(map('='.join, delta.items())))
             patch = {"spec": {"template": {"metadata": {"labels": delta}}}}
             patch_json = json.dumps(patch, indent=None)
+            controller_patch_target = self._get_controller_patch_target(controller)
             # NOTE: custom resources don't support strategic merge type. json merge is acceptable for both cases because the patch json doesn't contain lists
-            command = f"kubectl --namespace {self.config.namespace} patch {self.controller_type_name} {controller.name} --type='merge' -p '{patch_json}'"
+            command = f"kubectl --namespace {self.config.namespace} patch {controller_patch_target} --type='merge' -p '{patch_json}'"
             raise servo.checks.CheckError(
                 f"{self.controller_type_name} '{controller.name}' is missing labels: {desc}",
                 hint=f"Patch labels via: `{command}`",
@@ -642,7 +655,8 @@ class BaseOpsaniDevChecks(servo.BaseChecks, abc.ABC):
         )
         command = (
             f"kubectl exec -n {self.config.namespace} -c servo {self._servo_resource_target} -- "
-            f"servo --token-file /servo/opsani.token inject-sidecar --namespace {self.config.namespace} --service {self.config.service}{port_switch} "
+            f"servo --token-file /servo/opsani.token inject-sidecar --image {self.config.envoy_sidecar_image} "
+            f"--namespace {self.config.namespace} --service {self.config.service}{port_switch} "
             f"{self.controller_type_name.lower()}/{self.config_controller_name}"
         )
         raise servo.checks.CheckError(
@@ -827,7 +841,10 @@ class OpsaniDevChecks(BaseOpsaniDevChecks):
         return config.deployments[0]
 
     def _get_controller_service_selector(self, controller: servo.connectors.kubernetes.Deployment) -> Dict[str, str]:
-        return controller.obj.spec.selector.match_labels
+        return controller.match_labels
+
+    def _get_controller_patch_target(self, controller: servo.connectors.kubernetes.Deployment) -> str:
+        return f"Deployment {self.config_controller_name}"
 
 class OpsaniDevRolloutChecks(BaseOpsaniDevChecks):
     """Opsani dev checks against argoproj.io Rollouts"""
@@ -869,12 +886,18 @@ class OpsaniDevRolloutChecks(BaseOpsaniDevChecks):
         return config.rollouts[0]
 
     def _get_controller_service_selector(self, controller: servo.connectors.kubernetes.Rollout) -> Dict[str, str]:
-        match_labels = dict(controller.obj.spec.selector.match_labels)
+        match_labels = dict(controller.match_labels)
         assert controller.status, f"unable to determine service selector. rollout '{self.config.rollout}' in namespace '{self.config.namespace}' has no status"
         assert controller.status.current_pod_hash, f"unable to determine service selector. rollout '{self.config.rollout}' in namespace '{self.config.namespace}' has no currentPodHash"
         match_labels['rollouts-pod-template-hash'] = controller.status.current_pod_hash
         return match_labels
 
+    def _get_controller_patch_target(self, controller: servo.connectors.kubernetes.Rollout) -> str:
+        if controller.workload_ref_controller:
+            return f"{controller.workload_ref_controller.obj.kind} {controller.workload_ref_controller.name}"
+
+        return f"Rollout {self.config_controller_name}"
+
     @servo.checks.require("Rollout Selector and PodSpec has opsani_role label")
     async def check_rollout_selector_labels(self) -> None:
         if os.environ.get("POD_NAME") and os.environ.get("POD_NAMESPACE"):
@@ -884,7 +907,7 @@ class OpsaniDevRolloutChecks(BaseOpsaniDevChecks):
         assert rollout, f"failed to read Rollout '{self.config.rollout}' in namespace '{self.config.namespace}'"
 
         spec_patch = {}
-        match_labels = rollout.obj.spec.selector.match_labels or dict()
+        match_labels = rollout.match_labels or dict()
         opsani_role_selector = match_labels.get("opsani_role")
         if opsani_role_selector is None or opsani_role_selector == "tuning":
             opsani_role_selector = "mainline"
